import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _isEqual from "lodash/isEqual";
import invariant from 'invariant';
import React from 'react';
import { useFormState } from './FormState/Context';
import { deepEqual } from './FormState/utils';

/**
 * This hook works differently depending on whether it's inside a form state
 * context and whether it receives prop `fieldState`.
 *
 * Controlled (receiving prop `fieldState`): it will not have any local state
 * and just return the received `fieldState`.
 * This is when used like <Form><Form.Field component={MyField} /></Form>.
 *
 * Form Field (receiving prop `fieldState` and inside a form): it will connect
 * with the form and retrieve field state from the form state.
 * This is when used like <Form><MyField /></Form>.
 *
 * Standalone (not receiving prop `fieldState` and not inside a form): it will
 * have local field state and do validations by itself.
 * This is when used like <MyField />.
 */
export function useFieldState(props) {
  var fieldState = props.fieldState,
      name = props.name,
      _onBlur = props.onBlur,
      _onFocus = props.onFocus,
      _onValueChange = props.onValueChange,
      validate = props.validate,
      value = props.value;
  var formState = useFormState();
  var mode = fieldState ? 'controlled' : formState ? 'form-field' : 'standalone';
  /* Begin form field mode */

  invariant(!(mode === 'form-field' && !name), "prop \"name\" is required."); // register validators with form

  React.useEffect(function () {
    if (mode === 'form-field' && formState && name && validate) {
      formState.actions.setFieldValidators(name, validate);
    } // TODO: We currently don't support changing field name and validators on
    // the fly.
    // eslint-disable-next-line react-hooks/exhaustive-deps

  }, []);
  var formFieldModeReturnValue = React.useMemo(function () {
    if (mode !== 'form-field' || !formState || !name) return;
    var fieldActive = formState.fieldActive,
        fieldDirty = formState.fieldDirty,
        fieldPristine = formState.fieldPristine,
        fieldTouched = formState.fieldTouched,
        fieldVisited = formState.fieldVisited,
        getFieldError = formState.getFieldError,
        getFieldValue = formState.getFieldValue,
        actions = formState.actions;
    return {
      fieldState: {
        // field state
        active: fieldActive(name),
        dirty: fieldDirty(name),
        error: getFieldError(name),
        pristine: fieldPristine(name),
        touched: fieldTouched(name),
        value: getFieldValue(name),
        visited: fieldVisited(name),
        // form state
        formState: formState
      },
      onBlur: function onBlur() {
        _onBlur && _onBlur();
        actions.blur(name);
      },
      onFocus: function onFocus() {
        _onFocus && _onFocus();
        actions.focus(name);
      },
      onValueChange: function onValueChange(value) {
        _onValueChange && _onValueChange(value);
        actions.changeValue(name, value);
      }
    };
  }, [formState, mode, name, _onBlur, _onFocus, _onValueChange]);
  /* End form field mode */

  /* Begin standalone mode */

  var _React$useReducer = React.useReducer(createFieldStateReducer(), props, function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return getInitialStandaloneState.apply(void 0, [mode].concat(args));
  }),
      _React$useReducer2 = _slicedToArray(_React$useReducer, 2),
      standaloneState = _React$useReducer2[0],
      dispatch = _React$useReducer2[1]; // validate whenever value or validator changes


  React.useEffect(function () {
    if (mode === 'standalone' && (value !== prevValueRef.current || validate !== prevValidateRef.current)) {
      dispatch({
        type: 'CHANGE_VALUE_OR_VALIDATORS',
        newValue: value,
        newValidators: validate
      });
    }
  }, [mode, validate, value]); // save previous validator

  var prevValidateRef = React.useRef(validate);
  React.useEffect(function () {
    prevValidateRef.current = validate;
  }); // save previous value

  var prevValueRef = React.useRef(value);
  React.useEffect(function () {
    prevValueRef.current = value;
  });
  var standaloneModeReturnValue = React.useMemo(function () {
    if (mode !== 'standalone') return;
    return {
      fieldState: getFieldStateFromStandaloneState(value, standaloneState),
      onBlur: function onBlur() {
        dispatch({
          type: 'BLUR'
        });
        _onBlur && _onBlur();
      },
      onFocus: function onFocus() {
        dispatch({
          type: 'FOCUS'
        });
        _onFocus && _onFocus();
      },
      onValueChange: _onValueChange
    };
  }, [mode, _onBlur, _onFocus, _onValueChange, standaloneState, value]);
  /* End standalone mode */

  return mode === 'controlled' ? {
    fieldState: fieldState,
    onBlur: _onBlur,
    onFocus: _onFocus,
    onValueChange: _onValueChange
  } : mode === 'form-field' ? formFieldModeReturnValue : standaloneModeReturnValue;
}

function createFieldStateReducer() {
  return function fieldStateReducer(state, action) {
    switch (action.type) {
      case 'BLUR':
        return _objectSpread(_objectSpread({}, state), {}, {
          touched: true
        });

      case 'CHANGE_VALUE_OR_VALIDATORS':
        {
          var oldErrors = state.errors;

          var newState = _objectSpread(_objectSpread({}, state), {}, {
            errors: []
          });

          if (action.newValidators) {
            newState.errors = validate(getFieldStateFromStandaloneState(action.newValue, newState), action.newValidators);
          }

          if (_isEqual(oldErrors, newState.errors)) {
            // Errors didn't changed. Return old state to avoid rerender.
            return state;
          }

          return newState;
        }

      case 'FOCUS':
        return _objectSpread(_objectSpread({}, state), {}, {
          visited: true
        });

      default:
        return state;
    }
  };
}

function getInitialStandaloneState(mode, props) {
  var initialState = {
    errors: [],
    initialValue: props.value == null ? undefined : props.value,
    touched: false,
    visited: false
  };

  if (mode === 'standalone' && props.validate) {
    initialState.errors = validate(getFieldStateFromStandaloneState(props.value, initialState), props.validate);
  }

  return initialState;
}

function getFieldStateFromStandaloneState(value, _ref) {
  var errors = _ref.errors,
      initialValue = _ref.initialValue,
      touched = _ref.touched,
      visited = _ref.visited;
  var pristine = deepEqual(initialValue, value);
  return {
    dirty: !pristine,
    error: errors[0],
    pristine: pristine,
    touched: touched,
    value: value,
    visited: visited
  };
}

function validate(state, validators) {
  var errors = [];

  var validateField = function validateField(validator) {
    var error = validator(state.value, state);

    if (error) {
      errors.push(error);
    }
  };

  if (Array.isArray(validators)) {
    validators.forEach(validateField);
  } else {
    validateField(validators);
  }

  return errors;
}